import os
import os.path

from boto3.session import Session
import click
import toml

import notify


def get_profiles(config):
    if 'aws_profiles' not in config:
        return {
            'default': Session()
        }

    return {
        prof: Session(profile_name=prof)
        for prof in config['aws_profiles']
    }


def get_instance_statuses(ec2_client, config):
    def _filter(filters):
        _statuses = []
        next_token = ''

        while True:
            res = ec2_client.describe_instance_status(
                Filters=[
                    {'Name': k, 'Values': v}
                    for k, v in filters.iteritems()
                ],
                NextToken=next_token,
                MaxResults=1000
            )

            # We don't care about completed events. Thanks amazon for this
            # wonderful API.
            active_events = [
                event
                for status in res['InstanceStatuses']
                for event in status['Events']
                if not event['Description'].startswith('[Completed]')
            ]

            # Don't alert if we don't have any active events
            if len(active_events):
                _statuses.extend(res['InstanceStatuses'])
                next_token = res.get('NextToken')

            if not next_token:
                break

        return _statuses

    # describe_instance_status is an AND of filters, we want an OR
    seen = set()
    results = [
        _filter({'event.code': ['*']}),
        _filter({'instance-status.status': ['impaired']}),
        _filter({'instance-status.reachability': ['failed']}),
        _filter({'system-status.status': ['impaired']}),
        _filter({'system-status.reachability': ['failed']}),
    ]

    statuses = []
    for instances in results:
        statuses.extend([i for i in instances if i['InstanceId'] not in seen])
        seen.update([i['InstanceId'] for i in instances])

    return statuses


def get_config(config_locations):
    for loc in config_locations:
        file_name = os.path.expanduser(loc)

        if os.path.exists(file_name):
            click.echo('using config %s' % file_name, err=True)

            with open(file_name) as fp:
                return toml.loads(fp.read())


@click.command()
@click.option('--dry-run', is_flag=True, help='Disables sending alerts')
@click.option('--config', required=False, help='Configuration file location')
@click.option('--quiet', '-q', is_flag=True, help='Disables default JSON output')
def main(dry_run, config, quiet):
    config_names = [config] if config else ['pension.toml', '~/.pension.toml']

    try:
        config = get_config(config_names)
    except:
        click.echo('Failed to parse config', err=True)
        return -1

    if config is None:
        click.echo('No usable config file, trying environment vars', err=True)
        config = {'notify': {'json': {}}}

    data = {'instances': [], 'profiles': {}}
    instance_map = {}

    for prof, session in get_profiles(config).iteritems():
        ec2_client = session.client('ec2')
        ec2 = session.resource('ec2')

        statuses = get_instance_statuses(ec2_client, config)

        data['profiles'][prof] = {
            'region': session._session.get_config_variable('region'),
            'instances': [s['InstanceId'] for s in statuses]
        }
        data['instances'].extend(statuses)

        # Keep track of boto ec2 instances
        instances = ec2.instances.filter(InstanceIds=[
            s['InstanceId'] for s in statuses
        ])

        instance_map.update({
            statuses[i]['InstanceId']: inst
            for i, inst in enumerate(instances)
        })

    click.echo('%d instance(s) have reported issues' % len(data['instances']),
               err=True)

    if dry_run:
        config['notify'] = {} if quiet else {'json': {}}

    notify.send(data, instance_map, config['notify'])


if __name__ == '__main__':
    main()



import email
import json
import smtplib

from email.MIMEText import MIMEText

import requests


def json_serialize(obj):
    """handle datetime objects"""

    if hasattr(obj, 'isoformat'):
        return obj.isoformat()
    else:
        raise TypeError, 'Object of type %s with value of %s is not JSON \
serializable' % (type(obj), repr(obj))


def cleanup_statuses(statuses):
    """Downcase keys and only include relevant items"""

    def _format_item(item):
        status = {'status': item['Status']}
        status.update({
            e['Name']: e['Status'] for e in item['Details']
        })

        return status

    info = {}

    for status in statuses:
        instance = status['InstanceId']
        status = {
            'events': status.get('Events', []),
            'instance': _format_item(status['InstanceStatus']),
            'system': _format_item(status['SystemStatus']),
        }

        info[instance] = status

    return info


def send(data, inst_map, config):
    notify_fcns = {
        'json': notify_json,
        'slack': notify_slack,
        'email': notify_email,
    }

    # Don't notify unless there's actually something wrong
    if not len(data['instances']):
        return

    data['instances'] = cleanup_statuses(data['instances'])

    for key, fcn in notify_fcns.iteritems():
        if key in config:
            fcn(data, inst_map, config[key])


def notify_json(data, inst_map, config):
    if 'file' in config:
        with open(config['file'], 'w') as fp:
            json.dump(data, fp, indent=4, sort_keys=True, default=json_serialize)
    else:
        print json.dumps(data, indent=4, sort_keys=True, default=json_serialize)


def notify_slack(data, inst_map, config):
    console_url = '{inst_name} (<https://{region}.console.aws.amazon.com/ec2/v2/home?\
region={region}#Instances:search={instance}|{instance}>)'

    # This is kind of a gross API.
    tags = {
        inst_id: {t['Key']: t['Value'] for t in inst.tags}
        for inst_id, inst in inst_map.iteritems()
    }

    instance_links = [
        console_url.format(
            region=profile['region'],
            instance=inst_id,
            inst_name=tags[inst_id].get('Name', 'unnamed'))
        for profile in data['profiles'].values()
        for inst_id in profile['instances']
    ]

    post_data = {
        'text': '%d instance(s) have an issue: %s' % (
            len(data['instances']), ', '.join(instance_links))
    }

    # Add in the optional keys
    if 'user_name' in config:
        post_data['username'] = config['user_name']

    if 'channel' in config:
        post_data['channel'] = config['channel']

    res = requests.post(config['hook_url'], json=post_data)
    res.raise_for_status()


def notify_email(data, inst_map, config):

    msg_content = '''
    {num_instances} EC2 instances currently have issues!
    ========

    By AWS profile:
    --------

    {profile_counts}

    Full details:
    --------

    {details}
    '''.format(
        num_instances=len(data['instances']),
        profile_counts='\n'.join([
            '- [%s] %s: %s' % (data['region'], name, ', '.join(data['instances']))
            for name, data in data['profiles'].iteritems()
        ]),
        details=json.dumps(data, indent=4, sort_keys=True, default=json_serialize)
    )

    msg = MIMEText(msg_content, 'plain')
    msg['Subject'] = config['subject']
    msg['From'] = config.get('sender', config['user_name'])
    msg['To'] = ', '.join(config['recipients'])

    smtp = smtplib.SMTP_SSL(config['server'])
    smtp.login(config['user_name'], config['password'])
    smtp.sendmail(config['sender'], config['recipients'], msg.as_string())
    smtp.close()
